#include <windows.h>

// Application icon
IDI_APPLICATION ICON "../Assets/Logos/wpanda.ico"

// Version information
1 VERSIONINFO
FILEVERSION 4,3,0,0
PRODUCTVERSION 4,3,0,0
FILEFLAGSMASK VS_FFI_FILEFLAGSMASK
FILEFLAGS 0x0L
FILEOS VOS_NT_WINDOWS32
FILETYPE VFT_APP
FILESUBTYPE VFT2_UNKNOWN
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", "GIBIS-UNIFESP"
            VALUE "FileDescription", "wiRedPanda - Logic Circuit Simulator"
            VALUE "FileVersion", "4.3.0.0"
            VALUE "InternalName", "wiredpanda"
            VALUE "LegalCopyright", "GIBIS-UNIFESP and the wiRedPanda contributors"
            VALUE "OriginalFilename", "wiredpanda.exe"
            VALUE "ProductName", "wiRedPanda - Logic Circuit Simulator"
            VALUE "ProductVersion", "4.3.0.0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
