#!/usr/bin/env python3
# Copyright 2015 - 2026, GIBIS-UNIFESP and the wiRedPanda contributors
# SPDX-License-Identifier: GPL-3.0-or-later

"""Element spacing constants for IC generators."""

# Horizontal spacing for elements in a row
HORIZONTAL_GATE_SPACING = 104  # Between logic gate centers horizontally

# Vertical spacing between logic levels
VERTICAL_STAGE_SPACING = 104  # Between stages of logic (input → gates → output)
