// Copyright 2015 - 2026, GIBIS-UNIFESP and the wiRedPanda contributors
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <QObject>

class TestFiles : public QObject
{
    Q_OBJECT

private slots:
    void testFiles_data();
    void testFiles();
    void testBackwardCompatibility_data();
    void testBackwardCompatibility();
};
