// Copyright 2015 - 2026, GIBIS-UNIFESP and the wiRedPanda contributors
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <QObject>
#include <QTest>

class TestPropertyShortcutHandler : public QObject
{
    Q_OBJECT

private slots:

    void testGateInputCountNext();
    void testGateInputCountPrev();
    void testClockFrequencyNext();
    void testClockFrequencyPrev();
    void testBuzzerFrequency();
    void testLedColorSecondary();
    void testDisplayColorPrev();
    void testInputRotaryOutputSize();
    void testMorphNextElement();
    void testMorphPrevElement();
};
