// Copyright 2015 - 2026, GIBIS-UNIFESP and the wiRedPanda contributors
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <QObject>
#include <QTest>

class TestApplication : public QObject
{
    Q_OBJECT

private slots:

    void testApplicationNotify();
    void testExceptionHandling();

    // Regression: C10 — sentry deny-list filters known-noise patterns
    void testIsSentryDenyMessageMatchesClusterD();
    void testIsSentryDenyMessageDoesNotFilterUnknown();
};
