// Copyright 2015 - 2026, GIBIS-UNIFESP and the wiRedPanda contributors
// SPDX-License-Identifier: GPL-3.0-or-later

#include "Tests/Unit/Ui/TestElementPalette.h"

#include "Tests/Common/TestUtils.h"

void TestElementPalette::testPaletteSearch()
{
    // ElementPalette requires MainWindowUi — not available in unit tests
    QVERIFY(true);
}

void TestElementPalette::testPaletteRebuild()
{
    // ElementPalette requires MainWindowUi — not available in unit tests
    QVERIFY(true);
}
